@@ -320,6 +320,24 @@ static void config_check_module_duplicates (server *srv) {
 }
 
 __attribute_pure__
+__attribute_noinline__
+static int config_has_opt_enabled (const server * const srv, const char * const opt, const uint32_t olen) {
+    for (uint32_t i = 0; i < srv->config_context->used; ++i) {
+        const data_config * const config =
+          (const data_config *)srv->config_context->data[i];
+        const data_unset * const du =
+          array_get_data_unset(config->value, opt, olen);
+        if (NULL == du) continue;
+        if (du->type == TYPE_ARRAY
+            ? ((data_array *)du)->value.used != 0
+            : config_plugin_value_tobool(du, 0))
+            return 1;
+    }
+    return 0;
+}
+
+__attribute_pure__
+__attribute_noinline__
 static const char * config_has_opt_and_value (const server * const srv, const char * const opt, const uint32_t olen, const char * const v, const uint32_t vlen) {
     for (uint32_t i = 0; i < srv->config_context->used; ++i) {
         const data_config * const config =
@@ -333,6 +351,21 @@ static const char * config_has_opt_and_value (const server * const srv, const ch
     return NULL;
 }
 
+__attribute_noinline__
+static void config_compat_module_remove (server *srv, const char *module, uint32_t len) {
+    array *modules = array_init(srv->srvconf.modules->used);
+
+    for (uint32_t i = 0; i < srv->srvconf.modules->used; ++i) {
+        const data_string *ds = (data_string *)srv->srvconf.modules->data[i];
+        if (!buffer_eq_slen(&ds->value, module, len))
+            array_insert_value(modules, BUF_PTR_LEN(&ds->value));
+    }
+
+    array_free(srv->srvconf.modules);
+    srv->srvconf.modules = modules;
+}
+
+__attribute_noinline__
 static void config_compat_module_prepend (server *srv, const char *module, uint32_t len) {
     array *modules = array_init(srv->srvconf.modules->used+4);
     array_insert_value(modules, module, len);
@@ -433,6 +466,24 @@ static void config_compat_module_load (server *srv) {
         }
     }
 
+    /* check if some default modules are used and enabled
+     * (Each dynamically loaded modules takes at least 20k memory,
+     *  so avoid loading some default modules unless used and enabled) */
+
+    if (!config_has_opt_enabled(srv, CONST_STR_LEN("index-file.names"))
+        && !config_has_opt_enabled(srv, CONST_STR_LEN("server.indexfiles"))) {
+        if (!prepend_mod_indexfile)
+            config_compat_module_remove(srv, CONST_STR_LEN("mod_indexfile"));
+        prepend_mod_indexfile = 0;
+    }
+
+    if (!config_has_opt_enabled(srv, CONST_STR_LEN("dir-listing.activate"))
+        && !config_has_opt_enabled(srv, CONST_STR_LEN("server.dir-listing"))) {
+        if (!append_mod_dirlisting)
+            config_compat_module_remove(srv, CONST_STR_LEN("mod_dirlisting"));
+        append_mod_dirlisting = 0;
+    }
+
     /* prepend default modules */
 
     if (prepend_mod_indexfile) {
@@ -1035,8 +1086,7 @@ static int config_insert(server *srv) {
     };
 
     int rc = 0;
-    config_data_base * const p = calloc(1, sizeof(config_data_base));
-    force_assert(p);
+    config_data_base * const p = ck_calloc(1, sizeof(config_data_base));
     srv->config_data_base = p;
 
     if (!config_plugin_values_init(srv, p, cpk, "base"))
@@ -1120,8 +1170,7 @@ static int config_insert(server *srv) {
                     cpv->v.shrt |=FDEVENT_STREAM_RESPONSE;
                 break;
               case 18:{/*server.kbytes-per-second */
-                off_t * const cnt = malloc(2*sizeof(off_t));
-                force_assert(cnt);
+                off_t * const cnt = ck_malloc(2*sizeof(off_t));
                 cnt[0] = 0;
                 cnt[1] = (off_t)cpv->v.shrt << 10;
                 cpv->v.v = cnt;
@@ -1230,6 +1279,10 @@ int config_finalize(server *srv, const buffer *default_server_tag) {
             /* all var.* is known as user defined variable */
             if (strncmp(k->ptr, "var.", sizeof("var.") - 1) == 0)
                 continue;
+            /* mod_dirlisting not loaded if dir-listing.activate not enabled */
+            if (strncmp(k->ptr, "dir-listing.", sizeof("dir-listing.") - 1) == 0
+                && strcmp(k->ptr, "dir-listing.activate") != 0)
+                continue;
 
             if (!array_get_element_klen(srv->srvconf.config_touched,
                                         BUF_PTR_LEN(k)))
@@ -2386,12 +2439,14 @@ int config_remoteip_normalize(buffer * const b, buffer * const tb) {
 static void context_init(server *srv, config_t *context) {
 	context->srv = srv;
 	context->ok = 1;
-	vector_config_weak_init(&context->configs_stack);
+	context->configs_stack.data = NULL;
+	context->configs_stack.used = 0;
+	context->configs_stack.size = 0;
 	context->basedir = buffer_init();
 }
 
 static void context_free(config_t *context) {
-	vector_config_weak_clear(&context->configs_stack);
+	free(context->configs_stack.data);
 	buffer_free(context->basedir);
 }
 
