@@ -26,25 +26,35 @@ static data_config * configparser_get_data_config(const array *a, const char *k,
   return (data_config *)array_get_data_unset(a, k, klen);
 }
 
+__attribute_noinline__
+static void configparser_push_data_config_list(data_config_list *v, data_config *dc) {
+    if (v->size == v->used) {
+        ck_realloc_u32((void **)&v->data, v->size, 4, sizeof(*v->data));
+        v->size += 4;
+    }
+    v->data[v->used++] = dc;
+}
+
 static void configparser_push(config_t *ctx, data_config *dc, int isnew) {
   if (isnew) {
     dc->context_ndx = ctx->all_configs->used;
     force_assert(dc->context_ndx > ctx->current->context_ndx);
     array_insert_unique(ctx->all_configs, (data_unset *)dc);
     dc->parent = ctx->current;
-    vector_config_weak_push(&dc->parent->children, dc);
+    configparser_push_data_config_list(&dc->parent->children, dc);
   }
   if (ctx->configs_stack.used > 0 && ctx->current->context_ndx == 0) {
     fprintf(stderr, "Cannot use conditionals inside a global { ... } block\n");
     exit(-1);
   }
-  vector_config_weak_push(&ctx->configs_stack, ctx->current);
+  configparser_push_data_config_list(&ctx->configs_stack, ctx->current);
   ctx->current = dc;
 }
 
 static data_config *configparser_pop(config_t *ctx) {
   data_config *old = ctx->current;
-  ctx->current = vector_config_weak_pop(&ctx->configs_stack);
+  force_assert(ctx->configs_stack.used);
+  ctx->current = ctx->configs_stack.data[--ctx->configs_stack.used];
   force_assert(old && ctx->current);
   return old;
 }
@@ -341,7 +351,7 @@ configparser_parse_else_condition(config_t * const ctx)
 }
 
 
-#line 345 "./configparser.c"
+#line 355 "./configparser.c"
 /* Next is all token values, in a form suitable for use by makeheaders.
 ** This section will be null unless lemon is run with the -m switch.
 */
@@ -769,45 +779,45 @@ static void yy_destructor(YYCODETYPE yymajor, YYMINORTYPE *yypminor){
     case 26:
     case 27:
     case 28:
-#line 377 "../../src/configparser.y"
+#line 387 "../../src/configparser.y"
 { buffer_free((yypminor->yy0)); }
-#line 774 "./configparser.c"
-      break;
-    case 38:
-#line 368 "../../src/configparser.y"
-{ if ((yypminor->yy59)) (yypminor->yy59)->fn->free((yypminor->yy59)); }
-#line 779 "./configparser.c"
-      break;
-    case 39:
-#line 369 "../../src/configparser.y"
-{ if ((yypminor->yy59)) (yypminor->yy59)->fn->free((yypminor->yy59)); }
 #line 784 "./configparser.c"
       break;
-    case 40:
-#line 370 "../../src/configparser.y"
+    case 38:
+#line 378 "../../src/configparser.y"
 { if ((yypminor->yy59)) (yypminor->yy59)->fn->free((yypminor->yy59)); }
 #line 789 "./configparser.c"
       break;
-    case 43:
-#line 371 "../../src/configparser.y"
-{ array_free((yypminor->yy38)); }
+    case 39:
+#line 379 "../../src/configparser.y"
+{ if ((yypminor->yy59)) (yypminor->yy59)->fn->free((yypminor->yy59)); }
 #line 794 "./configparser.c"
       break;
-    case 44:
-#line 372 "../../src/configparser.y"
-{ array_free((yypminor->yy38)); }
+    case 40:
+#line 380 "../../src/configparser.y"
+{ if ((yypminor->yy59)) (yypminor->yy59)->fn->free((yypminor->yy59)); }
 #line 799 "./configparser.c"
       break;
-    case 45:
-#line 373 "../../src/configparser.y"
-{ buffer_free((yypminor->yy9)); }
+    case 43:
+#line 381 "../../src/configparser.y"
+{ array_free((yypminor->yy38)); }
 #line 804 "./configparser.c"
       break;
-    case 46:
-#line 374 "../../src/configparser.y"
-{ buffer_free((yypminor->yy9)); }
+    case 44:
+#line 382 "../../src/configparser.y"
+{ array_free((yypminor->yy38)); }
 #line 809 "./configparser.c"
       break;
+    case 45:
+#line 383 "../../src/configparser.y"
+{ buffer_free((yypminor->yy9)); }
+#line 814 "./configparser.c"
+      break;
+    case 46:
+#line 384 "../../src/configparser.y"
+{ buffer_free((yypminor->yy9)); }
+#line 819 "./configparser.c"
+      break;
     default:  break;   /* If no destructor action specified: do nothing */
   }
 }
@@ -1085,9 +1095,9 @@ static void yy_reduce(
         /* No destructor defined for global */
         break;
       case 5:
-#line 350 "../../src/configparser.y"
+#line 360 "../../src/configparser.y"
 { yymsp[-1].minor.yy18 = NULL; }
-#line 1090 "./configparser.c"
+#line 1100 "./configparser.c"
   yy_destructor(1,&yymsp[0].minor);
         break;
       case 6:
@@ -1100,7 +1110,7 @@ static void yy_reduce(
   yy_destructor(1,&yymsp[0].minor);
         break;
       case 9:
-#line 379 "../../src/configparser.y"
+#line 389 "../../src/configparser.y"
 {
   if (ctx->ok) {
     buffer_copy_buffer(&yymsp[0].minor.yy59->key, yymsp[-2].minor.yy9);
@@ -1124,11 +1134,11 @@ static void yy_reduce(
   if (yymsp[0].minor.yy59) yymsp[0].minor.yy59->fn->free(yymsp[0].minor.yy59);
   yymsp[0].minor.yy59 = NULL;
 }
-#line 1127 "./configparser.c"
+#line 1137 "./configparser.c"
   yy_destructor(2,&yymsp[-1].minor);
         break;
       case 10:
-#line 403 "../../src/configparser.y"
+#line 413 "../../src/configparser.y"
 {
   if (ctx->ok) {
     if (strncmp(yymsp[-2].minor.yy9->ptr, "env.", sizeof("env.") - 1) == 0) {
@@ -1147,11 +1157,11 @@ static void yy_reduce(
   if (yymsp[0].minor.yy59) yymsp[0].minor.yy59->fn->free(yymsp[0].minor.yy59);
   yymsp[0].minor.yy59 = NULL;
 }
-#line 1150 "./configparser.c"
+#line 1160 "./configparser.c"
   yy_destructor(3,&yymsp[-1].minor);
         break;
       case 11:
-#line 422 "../../src/configparser.y"
+#line 432 "../../src/configparser.y"
 {
   if (ctx->ok) {
     array *vars = ctx->current->value;
@@ -1182,11 +1192,11 @@ static void yy_reduce(
   if (yymsp[0].minor.yy59) yymsp[0].minor.yy59->fn->free(yymsp[0].minor.yy59);
   yymsp[0].minor.yy59 = NULL;
 }
-#line 1185 "./configparser.c"
+#line 1195 "./configparser.c"
   yy_destructor(4,&yymsp[-1].minor);
         break;
       case 12:
-#line 453 "../../src/configparser.y"
+#line 463 "../../src/configparser.y"
 {
   if (strchr(yymsp[0].minor.yy0->ptr, '.') == NULL) {
     buffer_copy_string((yygotominor.yy9 = buffer_init()), "var.");
@@ -1198,10 +1208,10 @@ static void yy_reduce(
   buffer_free(yymsp[0].minor.yy0);
   yymsp[0].minor.yy0 = NULL;
 }
-#line 1201 "./configparser.c"
+#line 1211 "./configparser.c"
         break;
       case 13:
-#line 465 "../../src/configparser.y"
+#line 475 "../../src/configparser.y"
 {
   yygotominor.yy59 = NULL;
   if (ctx->ok) {
@@ -1216,19 +1226,19 @@ static void yy_reduce(
   if (yymsp[0].minor.yy59) yymsp[0].minor.yy59->fn->free(yymsp[0].minor.yy59);
   yymsp[0].minor.yy59 = NULL;
 }
-#line 1219 "./configparser.c"
+#line 1229 "./configparser.c"
   yy_destructor(6,&yymsp[-1].minor);
         break;
       case 14:
-#line 480 "../../src/configparser.y"
+#line 490 "../../src/configparser.y"
 {
   yygotominor.yy59 = yymsp[0].minor.yy59;
   yymsp[0].minor.yy59 = NULL;
 }
-#line 1228 "./configparser.c"
+#line 1238 "./configparser.c"
         break;
       case 15:
-#line 485 "../../src/configparser.y"
+#line 495 "../../src/configparser.y"
 {
   yygotominor.yy59 = NULL;
   if (ctx->ok) {
@@ -1253,10 +1263,10 @@ static void yy_reduce(
   buffer_free(yymsp[0].minor.yy9);
   yymsp[0].minor.yy9 = NULL;
 }
-#line 1256 "./configparser.c"
+#line 1266 "./configparser.c"
         break;
       case 16:
-#line 510 "../../src/configparser.y"
+#line 520 "../../src/configparser.y"
 {
   yygotominor.yy59 = (data_unset *)array_data_string_init();
   /* assumes array_data_string_init() result does not need swap, buffer_free()*/
@@ -1264,10 +1274,10 @@ static void yy_reduce(
   free(yymsp[0].minor.yy0);
   yymsp[0].minor.yy0 = NULL;
 }
-#line 1267 "./configparser.c"
+#line 1277 "./configparser.c"
         break;
       case 17:
-#line 518 "../../src/configparser.y"
+#line 528 "../../src/configparser.y"
 {
   char *endptr;
   yygotominor.yy59 = (data_unset *)array_data_integer_init();
@@ -1282,10 +1292,10 @@ static void yy_reduce(
   buffer_free(yymsp[0].minor.yy0);
   yymsp[0].minor.yy0 = NULL;
 }
-#line 1285 "./configparser.c"
+#line 1295 "./configparser.c"
         break;
       case 18:
-#line 532 "../../src/configparser.y"
+#line 542 "../../src/configparser.y"
 {
   yygotominor.yy59 = (data_unset *)array_data_array_init();
   /* assumes array_data_array_init() result does not need swap, array_free() */
@@ -1293,29 +1303,29 @@ static void yy_reduce(
   free(yymsp[0].minor.yy38);
   yymsp[0].minor.yy38 = NULL;
 }
-#line 1296 "./configparser.c"
+#line 1306 "./configparser.c"
         break;
       case 19:
-#line 539 "../../src/configparser.y"
+#line 549 "../../src/configparser.y"
 {
   yygotominor.yy38 = array_init(8);
 }
-#line 1303 "./configparser.c"
+#line 1313 "./configparser.c"
   yy_destructor(9,&yymsp[-1].minor);
   yy_destructor(10,&yymsp[0].minor);
         break;
       case 20:
-#line 542 "../../src/configparser.y"
+#line 552 "../../src/configparser.y"
 {
   yygotominor.yy38 = yymsp[-1].minor.yy38;
   yymsp[-1].minor.yy38 = NULL;
 }
-#line 1313 "./configparser.c"
+#line 1323 "./configparser.c"
   yy_destructor(9,&yymsp[-2].minor);
   yy_destructor(10,&yymsp[0].minor);
         break;
       case 21:
-#line 547 "../../src/configparser.y"
+#line 557 "../../src/configparser.y"
 {
   yygotominor.yy38 = NULL;
   if (ctx->ok) {
@@ -1337,20 +1347,20 @@ static void yy_reduce(
   if (yymsp[0].minor.yy59) yymsp[0].minor.yy59->fn->free(yymsp[0].minor.yy59);
   yymsp[0].minor.yy59 = NULL;
 }
-#line 1340 "./configparser.c"
+#line 1350 "./configparser.c"
   yy_destructor(11,&yymsp[-1].minor);
         break;
       case 22:
-#line 569 "../../src/configparser.y"
+#line 579 "../../src/configparser.y"
 {
   yygotominor.yy38 = yymsp[-1].minor.yy38;
   yymsp[-1].minor.yy38 = NULL;
 }
-#line 1349 "./configparser.c"
+#line 1359 "./configparser.c"
   yy_destructor(11,&yymsp[0].minor);
         break;
       case 23:
-#line 574 "../../src/configparser.y"
+#line 584 "../../src/configparser.y"
 {
   yygotominor.yy38 = NULL;
   if (ctx->ok) {
@@ -1361,18 +1371,18 @@ static void yy_reduce(
   if (yymsp[0].minor.yy59) yymsp[0].minor.yy59->fn->free(yymsp[0].minor.yy59);
   yymsp[0].minor.yy59 = NULL;
 }
-#line 1364 "./configparser.c"
+#line 1374 "./configparser.c"
         break;
       case 24:
-#line 585 "../../src/configparser.y"
+#line 595 "../../src/configparser.y"
 {
   yygotominor.yy59 = yymsp[0].minor.yy59;
   yymsp[0].minor.yy59 = NULL;
 }
-#line 1372 "./configparser.c"
+#line 1382 "./configparser.c"
         break;
       case 25:
-#line 589 "../../src/configparser.y"
+#line 599 "../../src/configparser.y"
 {
   yygotominor.yy59 = NULL;
   if (ctx->ok) {
@@ -1386,7 +1396,7 @@ static void yy_reduce(
   buffer_free(yymsp[-2].minor.yy9);
   yymsp[-2].minor.yy9 = NULL;
 }
-#line 1389 "./configparser.c"
+#line 1399 "./configparser.c"
   yy_destructor(12,&yymsp[-1].minor);
         break;
       case 26:
@@ -1395,29 +1405,29 @@ static void yy_reduce(
       case 27:
         break;
       case 28:
-#line 606 "../../src/configparser.y"
+#line 616 "../../src/configparser.y"
 {
   data_config *dc;
   dc = configparser_get_data_config(ctx->srv->config_context, CONST_STR_LEN("global"));
   force_assert(dc);
   configparser_push(ctx, dc, 0);
 }
-#line 1405 "./configparser.c"
+#line 1415 "./configparser.c"
   yy_destructor(13,&yymsp[0].minor);
         break;
       case 29:
-#line 613 "../../src/configparser.y"
+#line 623 "../../src/configparser.y"
 {
   configparser_pop(ctx);
 }
-#line 1413 "./configparser.c"
+#line 1423 "./configparser.c"
         /* No destructor defined for globalstart */
   yy_destructor(14,&yymsp[-2].minor);
         /* No destructor defined for metalines */
   yy_destructor(15,&yymsp[0].minor);
         break;
       case 30:
-#line 617 "../../src/configparser.y"
+#line 627 "../../src/configparser.y"
 {
   yygotominor.yy18 = NULL;
   if (ctx->ok) {
@@ -1436,12 +1446,12 @@ static void yy_reduce(
   yymsp[-3].minor.yy18 = NULL;
   yymsp[0].minor.yy18 = NULL;
 }
-#line 1439 "./configparser.c"
+#line 1449 "./configparser.c"
         /* No destructor defined for eols */
   yy_destructor(16,&yymsp[-1].minor);
         break;
       case 31:
-#line 636 "../../src/configparser.y"
+#line 646 "../../src/configparser.y"
 {
   yygotominor.yy18 = NULL;
   if (ctx->ok) {
@@ -1511,48 +1521,48 @@ static void yy_reduce(
   yymsp[-3].minor.yy18 = NULL;
   yymsp[0].minor.yy18 = NULL;
 }
-#line 1514 "./configparser.c"
+#line 1524 "./configparser.c"
         /* No destructor defined for eols */
   yy_destructor(16,&yymsp[-1].minor);
         break;
       case 32:
-#line 706 "../../src/configparser.y"
+#line 716 "../../src/configparser.y"
 {
   yygotominor.yy18 = yymsp[0].minor.yy18;
   yymsp[0].minor.yy18 = NULL;
 }
-#line 1524 "./configparser.c"
+#line 1534 "./configparser.c"
         break;
       case 33:
-#line 711 "../../src/configparser.y"
+#line 721 "../../src/configparser.y"
 {
   yygotominor.yy18 = NULL;
   if (ctx->ok) {
     yygotominor.yy18 = configparser_pop(ctx);
   }
 }
-#line 1534 "./configparser.c"
+#line 1544 "./configparser.c"
         /* No destructor defined for context */
   yy_destructor(14,&yymsp[-2].minor);
         /* No destructor defined for metalines */
   yy_destructor(15,&yymsp[0].minor);
         break;
       case 34:
-#line 718 "../../src/configparser.y"
+#line 728 "../../src/configparser.y"
 {
   yygotominor.yy18 = NULL;
   if (ctx->ok) {
     yygotominor.yy18 = configparser_pop(ctx);
   }
 }
-#line 1548 "./configparser.c"
+#line 1558 "./configparser.c"
         /* No destructor defined for context_else */
   yy_destructor(14,&yymsp[-2].minor);
         /* No destructor defined for metalines */
   yy_destructor(15,&yymsp[0].minor);
         break;
       case 35:
-#line 725 "../../src/configparser.y"
+#line 735 "../../src/configparser.y"
 {
 
   if (ctx->ok && yymsp[0].minor.yy59->type != TYPE_STRING) {
@@ -1568,73 +1578,73 @@ static void yy_reduce(
   yymsp[-5].minor.yy0 = NULL;
   buffer_free(yymsp[-3].minor.yy9);
   yymsp[-3].minor.yy9 = NULL;
-  yymsp[0].minor.yy59->fn->free(yymsp[0].minor.yy59);
+  if (yymsp[0].minor.yy59) yymsp[0].minor.yy59->fn->free(yymsp[0].minor.yy59);
   yymsp[0].minor.yy59 = NULL;
 }
-#line 1574 "./configparser.c"
+#line 1584 "./configparser.c"
   yy_destructor(17,&yymsp[-6].minor);
   yy_destructor(19,&yymsp[-4].minor);
   yy_destructor(20,&yymsp[-2].minor);
         break;
       case 36:
-#line 744 "../../src/configparser.y"
+#line 754 "../../src/configparser.y"
 {
   if (ctx->ok) {
     configparser_parse_else_condition(ctx);
   }
 }
-#line 1586 "./configparser.c"
+#line 1596 "./configparser.c"
         break;
       case 37:
-#line 750 "../../src/configparser.y"
+#line 760 "../../src/configparser.y"
 {
   yygotominor.yy69 = CONFIG_COND_EQ;
 }
-#line 1593 "./configparser.c"
+#line 1603 "./configparser.c"
   yy_destructor(21,&yymsp[0].minor);
         break;
       case 38:
-#line 753 "../../src/configparser.y"
+#line 763 "../../src/configparser.y"
 {
   yygotominor.yy69 = CONFIG_COND_MATCH;
 }
-#line 1601 "./configparser.c"
+#line 1611 "./configparser.c"
   yy_destructor(22,&yymsp[0].minor);
         break;
       case 39:
-#line 756 "../../src/configparser.y"
+#line 766 "../../src/configparser.y"
 {
   yygotominor.yy69 = CONFIG_COND_NE;
 }
-#line 1609 "./configparser.c"
+#line 1619 "./configparser.c"
   yy_destructor(23,&yymsp[0].minor);
         break;
       case 40:
-#line 759 "../../src/configparser.y"
+#line 769 "../../src/configparser.y"
 {
   yygotominor.yy69 = CONFIG_COND_NOMATCH;
 }
-#line 1617 "./configparser.c"
+#line 1627 "./configparser.c"
   yy_destructor(24,&yymsp[0].minor);
         break;
       case 41:
-#line 762 "../../src/configparser.y"
+#line 772 "../../src/configparser.y"
 {
   yygotominor.yy69 = CONFIG_COND_PREFIX;
 }
-#line 1625 "./configparser.c"
+#line 1635 "./configparser.c"
   yy_destructor(25,&yymsp[0].minor);
         break;
       case 42:
-#line 765 "../../src/configparser.y"
+#line 775 "../../src/configparser.y"
 {
   yygotominor.yy69 = CONFIG_COND_SUFFIX;
 }
-#line 1633 "./configparser.c"
+#line 1643 "./configparser.c"
   yy_destructor(26,&yymsp[0].minor);
         break;
       case 43:
-#line 769 "../../src/configparser.y"
+#line 779 "../../src/configparser.y"
 {
   yygotominor.yy9 = NULL;
   if (ctx->ok) {
@@ -1651,10 +1661,10 @@ static void yy_reduce(
   if (yymsp[0].minor.yy59) yymsp[0].minor.yy59->fn->free(yymsp[0].minor.yy59);
   yymsp[0].minor.yy59 = NULL;
 }
-#line 1654 "./configparser.c"
+#line 1664 "./configparser.c"
         break;
       case 44:
-#line 786 "../../src/configparser.y"
+#line 796 "../../src/configparser.y"
 {
   if (ctx->ok) {
     if (0 != config_parse_file(ctx->srv, ctx, yymsp[0].minor.yy9->ptr)) {
@@ -1664,11 +1674,11 @@ static void yy_reduce(
   buffer_free(yymsp[0].minor.yy9);
   yymsp[0].minor.yy9 = NULL;
 }
-#line 1667 "./configparser.c"
+#line 1677 "./configparser.c"
   yy_destructor(27,&yymsp[-1].minor);
         break;
       case 45:
-#line 796 "../../src/configparser.y"
+#line 806 "../../src/configparser.y"
 {
   if (ctx->ok) {
     if (0 != config_parse_cmd(ctx->srv, ctx, yymsp[0].minor.yy9->ptr)) {
@@ -1678,7 +1688,7 @@ static void yy_reduce(
   buffer_free(yymsp[0].minor.yy9);
   yymsp[0].minor.yy9 = NULL;
 }
-#line 1681 "./configparser.c"
+#line 1691 "./configparser.c"
   yy_destructor(28,&yymsp[-1].minor);
         break;
   };
@@ -1708,11 +1718,11 @@ static void yy_parse_failed(
   while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
   /* Here code is inserted which will be executed whenever the
   ** parser fails */
-#line 341 "../../src/configparser.y"
+#line 351 "../../src/configparser.y"
 
   ctx->ok = 0;
 
-#line 1715 "./configparser.c"
+#line 1725 "./configparser.c"
   configparserARG_STORE; /* Suppress warning about unused %extra_argument variable */
 }
 
