@@ -18,6 +18,7 @@
 #include "buffer.h"
 #include "fdevent.h"
 #include "http_chunk.h"
+#include "http_etag.h"
 #include "http_header.h"
 #include "keyvalue.h"
 #include "response.h"
@@ -109,10 +110,10 @@ typedef struct {
 typedef struct {
 	dirls_entry_t **ent;
 	uint32_t used;
-	uint32_t size;
 } dirls_list_t;
 
 #define DIRLIST_ENT_NAME(ent)  ((char*)(ent) + sizeof(dirls_entry_t))
+/* DIRLIST_BLOB_SIZE must be power of 2 for current internal usage */
 #define DIRLIST_BLOB_SIZE      16
 
 typedef struct {
@@ -136,8 +137,7 @@ static int dirlist_max_in_progress;
 
 
 static handler_ctx * mod_dirlisting_handler_ctx_init (plugin_data * const p) {
-    handler_ctx *hctx = calloc(1, sizeof(*hctx));
-    force_assert(hctx);
+    handler_ctx *hctx = ck_calloc(1, sizeof(*hctx));
     memcpy(&hctx->conf, &p->conf, sizeof(plugin_config));
     return hctx;
 }
@@ -209,8 +209,8 @@ static struct dirlist_cache * mod_dirlisting_parse_cache(server *srv, const arra
         }
     }
 
-    struct dirlist_cache * const cache = calloc(1, sizeof(struct dirlist_cache));
-    force_assert(cache);
+    struct dirlist_cache * const cache =
+      ck_calloc(1, sizeof(struct dirlist_cache));
     cache->max_age = max_age;
     cache->path = path;
     return cache;
@@ -260,7 +260,7 @@ static int mod_dirlisting_exclude(pcre_keyvalue_buffer * const kvb, const char *
 
 
 INIT_FUNC(mod_dirlisting_init) {
-    return calloc(1, sizeof(plugin_data));
+    return ck_calloc(1, sizeof(plugin_data));
 }
 
 FREE_FUNC(mod_dirlisting_free) {
@@ -995,8 +995,7 @@ static int http_open_directory(request_st * const r, handler_ctx * const hctx) {
     /* allocate based on PATH_MAX rather than pathconf() to get _PC_NAME_MAX */
     hctx->name_max = PATH_MAX - dlen - 1;
 #endif
-    hctx->path = malloc(dlen + hctx->name_max + 1);
-    force_assert(NULL != hctx->path);
+    hctx->path = ck_malloc(dlen + hctx->name_max + 1);
     memcpy(hctx->path, r->physical.path.ptr, dlen+1);
   #if defined(HAVE_XATTR) || defined(HAVE_EXTATTR) || !defined(_ATFILE_SOURCE)
     hctx->path_file = hctx->path + dlen;
@@ -1022,15 +1021,9 @@ static int http_open_directory(request_st * const r, handler_ctx * const hctx) {
 
     dirls_list_t * const dirs = &hctx->dirs;
     dirls_list_t * const files = &hctx->files;
-    dirs->ent   =
-      (dirls_entry_t**) malloc(sizeof(dirls_entry_t*) * DIRLIST_BLOB_SIZE);
-    force_assert(dirs->ent);
-    dirs->size  = DIRLIST_BLOB_SIZE;
+    dirs->ent   = NULL;
     dirs->used  = 0;
-    files->ent  =
-      (dirls_entry_t**) malloc(sizeof(dirls_entry_t*) * DIRLIST_BLOB_SIZE);
-    force_assert(files->ent);
-    files->size = DIRLIST_BLOB_SIZE;
+    files->ent  = NULL;
     files->used = 0;
 
     return 0;
@@ -1098,7 +1091,7 @@ static int http_read_directory(handler_ctx * const p) {
 				p->jcomma = 1;
 				buffer_append_string_len(p->jb, CONST_STR_LEN( "{\"name\":\""));
 			}
-			buffer_append_string_encoded_json(p->jb, d_name, dsz);
+			buffer_append_bs_escaped_json(p->jb, d_name, dsz);
 
 			const char *t;
 			size_t tlen;
@@ -1124,14 +1117,11 @@ static int http_read_directory(handler_ctx * const p) {
 		}
 
 		dirls_list_t * const list = !S_ISDIR(st.st_mode) ? &p->files : &p->dirs;
-		if (list->used == list->size) {
-			list->size += DIRLIST_BLOB_SIZE;
-			list->ent   = (dirls_entry_t**) realloc(list->ent, sizeof(dirls_entry_t*) * list->size);
-			force_assert(list->ent);
-		}
+		if (!(list->used & (DIRLIST_BLOB_SIZE-1)))
+			ck_realloc_u32((void **)&list->ent, list->used,
+			               DIRLIST_BLOB_SIZE, sizeof(*list->ent));
 		dirls_entry_t * const tmp = list->ent[list->used++] =
-		  (dirls_entry_t*) malloc(sizeof(dirls_entry_t) + 1 + dsz);
-		force_assert(tmp);
+		  (dirls_entry_t*) ck_malloc(sizeof(dirls_entry_t) + 1 + dsz);
 		tmp->mtime = st.st_mtime;
 		tmp->size  = st.st_size;
 		tmp->namelen = dsz;
@@ -1260,7 +1250,7 @@ static void mod_dirlisting_content_type (request_st * const r, const buffer * co
     if (NULL == encoding)
         buffer_copy_string_len(vb, CONST_STR_LEN("text/html"));
     else
-        buffer_append_str2(vb, CONST_STR_LEN("text/html; charset="),
+        buffer_append_str2(vb, CONST_STR_LEN("text/html;charset="),
                                BUF_PTR_LEN(encoding));
 }
 
@@ -1462,9 +1452,20 @@ REQUEST_FUNC(mod_dirlisting_reset) {
 }
 
 
+static void mod_dirlisting_cache_control (request_st * const r, unix_time64_t max_age) {
+    if (!light_btst(r->resp_htags, HTTP_HEADER_CACHE_CONTROL)) {
+        buffer * const vb =
+          http_header_response_set_ptr(r, HTTP_HEADER_CACHE_CONTROL,
+                                       CONST_STR_LEN("Cache-Control"));
+        buffer_append_string_len(vb, CONST_STR_LEN("max-age="));
+        buffer_append_int(vb, max_age);
+    }
+}
+
+
 static handler_t mod_dirlisting_cache_check (request_st * const r, plugin_data * const p) {
     /* optional: an external process can trigger a refresh by deleting the cache
-     * entry when the external process detects (or initiaties) changes to dir */
+     * entry when the external process detects (or initiates) changes to dir */
     buffer * const tb = r->tmp_buf;
     buffer_copy_path_len2(tb, BUF_PTR_LEN(p->conf.cache->path),
                               BUF_PTR_LEN(&r->physical.path));
@@ -1473,7 +1474,9 @@ static handler_t mod_dirlisting_cache_check (request_st * const r, plugin_data *
     stat_cache_entry * const sce = stat_cache_get_entry_open(tb, 1);
     if (NULL == sce || sce->fd == -1)
         return HANDLER_GO_ON;
-    if (TIME64_CAST(sce->st.st_mtime) + p->conf.cache->max_age < log_epoch_secs)
+    const unix_time64_t max_age =
+      TIME64_CAST(sce->st.st_mtime) + p->conf.cache->max_age - log_epoch_secs;
+    if (max_age < 0)
         return HANDLER_GO_ON;
 
     !p->conf.json
@@ -1505,6 +1508,16 @@ static handler_t mod_dirlisting_cache_check (request_st * const r, plugin_data *
         return HANDLER_GO_ON;
     }
 
+    /* Cache-Control and ETag (also done in mod_dirlisting_cache_add())*/
+    mod_dirlisting_cache_control(r, max_age);
+    if (0 != r->conf.etag_flags) {
+        const buffer *etag = stat_cache_etag_get(sce, r->conf.etag_flags);
+        if (etag && !buffer_is_blank(etag))
+            http_header_response_set(r, HTTP_HEADER_ETAG,
+                                     CONST_STR_LEN("ETag"),
+                                     BUF_PTR_LEN(etag));
+    }
+
     r->resp_body_finished = 1;
     return HANDLER_FINISHED;
 }
@@ -1519,16 +1532,17 @@ static int mod_dirlisting_write_cq (const int fd, chunkqueue * const cq, log_err
     cq->bytes_out -= in.bytes_in;
 
     /*(similar to mod_webdav.c:mod_webdav_write_cq(), but operates on two cqs)*/
-    chunkqueue_remove_finished_chunks(&in);
     while (!chunkqueue_is_empty(&in)) {
         ssize_t wr = chunkqueue_write_chunk(fd, &in, errh);
-        if (wr > 0)
+        if (__builtin_expect( (wr > 0), 1))
             chunkqueue_steal(cq, &in, wr);
         else if (wr < 0) {
             /*(writing to tempfile failed; transfer remaining data back to cq)*/
             chunkqueue_append_chunkqueue(cq, &in);
             return 0;
         }
+        else /*(wr == 0)*/
+            chunkqueue_remove_finished_chunks(&in);
     }
     return 1;
 }
@@ -1580,8 +1594,20 @@ static void mod_dirlisting_cache_add (request_st * const r, handler_ctx * const
     const int fd = fdevent_mkostemp(oldpath, 0);
     if (fd < 0) return;
     if (mod_dirlisting_write_cq(fd, &r->write_queue, r->conf.errh)
-        && 0 == rename(oldpath, newpath))
+        && 0 == rename(oldpath, newpath)) {
         stat_cache_invalidate_entry(newpath, len);
+        /* Cache-Control and ETag (also done in mod_dirlisting_cache_check())*/
+        mod_dirlisting_cache_control(r, hctx->conf.cache->max_age);
+        if (0 != r->conf.etag_flags) {
+            struct stat st;
+            if (0 == fstat(fd, &st)) {
+                buffer * const vb =
+                  http_header_response_set_ptr(r, HTTP_HEADER_ETAG,
+                                               CONST_STR_LEN("ETag"));
+                http_etag_create(vb, &st, r->conf.etag_flags);
+            }
+        }
+    }
     else
         unlink(oldpath);
     close(fd);
@@ -1604,8 +1630,7 @@ static void mod_dirlisting_cache_json_init (request_st * const r, handler_ctx *
     if (fd < 0) return;
     hctx->jfn_len = buffer_clen(tb);
     hctx->jfd = fd;
-    hctx->jfn = malloc(hctx->jfn_len+1);
-    force_assert(hctx->jfn);
+    hctx->jfn = ck_malloc(hctx->jfn_len+1);
     memcpy(hctx->jfn, tb->ptr, hctx->jfn_len+1); /*(include '\0')*/
 }
 
@@ -1630,6 +1655,7 @@ static void mod_dirlisting_cache_json (request_st * const r, handler_ctx * const
 }
 
 
+__attribute_cold__
 int mod_dirlisting_plugin_init(plugin *p);
 int mod_dirlisting_plugin_init(plugin *p) {
 	p->version     = LIGHTTPD_VERSION_ID;
