@@ -68,7 +68,7 @@ typedef struct {
 
 
 INIT_FUNC(mod_proxy_init) {
-    return calloc(1, sizeof(plugin_data));
+    return ck_calloc(1, sizeof(plugin_data));
 }
 
 
@@ -264,8 +264,7 @@ static http_header_remap_opts * mod_proxy_parse_header_opts(server *srv, const a
         }
     }
 
-    http_header_remap_opts *opts = malloc(sizeof(header));
-    force_assert(opts);
+    http_header_remap_opts *opts = ck_malloc(sizeof(header));
     memcpy(opts, &header, sizeof(header));
     return opts;
 }
@@ -312,8 +311,7 @@ SETDEFAULTS_FUNC(mod_proxy_set_defaults)
         for (; -1 != cpv->k_id; ++cpv) {
             switch (cpv->k_id) {
               case 0: /* proxy.server */
-                gw = calloc(1, sizeof(gw_plugin_config));
-                force_assert(gw);
+                gw = ck_calloc(1, sizeof(gw_plugin_config));
                 if (!gw_set_defaults_backend(srv, (gw_plugin_data *)p, cpv->v.a,
                                              gw, 0, cpk[cpv->k_id].k)) {
                     gw_plugin_config_free(gw);
@@ -1013,7 +1011,7 @@ static handler_t proxy_create_env(gw_handler_ctx *gwhctx) {
 		if (!http_header_request_get(r, HTTP_HEADER_OTHER,
 		                             CONST_STR_LEN("Sec-WebSocket-Key")))
 			buffer_append_string_len(b, CONST_STR_LEN(
-			  "\r\nSec-WebSocket-Key: MDAwMDAwMDAwMDAwMDAwMAo="));
+			  "\r\nSec-WebSocket-Key: MDAwMDAwMDAwMDAwMDAwMA=="));
 		buffer_append_string_len(b, CONST_STR_LEN(
 		                              "\r\nUpgrade: websocket"
 		                              "\r\nConnection: close, upgrade\r\n\r\n"));
@@ -1173,6 +1171,7 @@ static handler_t mod_proxy_check_extension(request_st * const r, void *p_d) {
 }
 
 
+__attribute_cold__
 int mod_proxy_plugin_init(plugin *p);
 int mod_proxy_plugin_init(plugin *p) {
 	p->version      = LIGHTTPD_VERSION_ID;
