@@ -41,7 +41,7 @@ typedef struct {
 } plugin_data;
 
 INIT_FUNC(mod_status_init) {
-    return calloc(1, sizeof(plugin_data));
+    return ck_calloc(1, sizeof(plugin_data));
 }
 
 static void mod_status_merge_config_cpv(plugin_config * const pconf, const config_plugin_value_t * const cpv) {
@@ -844,6 +844,7 @@ REQUESTDONE_FUNC(mod_status_account) {
 }
 
 
+__attribute_cold__
 int mod_status_plugin_init(plugin *p);
 int mod_status_plugin_init(plugin *p) {
 	p->version     = LIGHTTPD_VERSION_ID;
