@@ -773,7 +773,8 @@ http_response_write_prepare(request_st * const r)
 
     if (r->resp_body_finished) {
         /* check for Range request (current impl requires resp_body_finished) */
-        if (r->conf.range_requests && http_range_rfc7233(r) >= 400)
+        if (r->conf.range_requests && r->http_status == 200
+            && http_range_rfc7233(r) >= 400)
             http_response_static_errdoc(r); /* 416 Range Not Satisfiable */
 
         /* set content-length if length is known and not already set */
@@ -905,12 +906,8 @@ http_response_call_error_handler (request_st * const r, const buffer * const err
         }
 
         r->con->is_writable = 1;
-        r->resp_body_finished = 0;
-        r->resp_body_started = 0;
-
         r->error_handler_saved_status = r->http_status;
         r->error_handler_saved_method = r->http_method;
-
         r->http_method = HTTP_METHOD_GET;
     }
     else { /*(preserve behavior for server.error-handler-404)*/
