@@ -5,7 +5,6 @@
 
 void test_mod_access (void);
 void test_mod_alias (void);
-void test_mod_evasive (void);
 void test_mod_evhost (void);
 void test_mod_indexfile (void);
 void test_mod_simple_vhost (void);
@@ -16,7 +15,6 @@ void test_mod_userdir (void);
 int main() {
     test_mod_access();
     test_mod_alias();
-    test_mod_evasive();
     test_mod_evhost();
     test_mod_indexfile();
     test_mod_simple_vhost();
